import { EntityInitializationCause, MolangVariableMap, Player, WeatherType, system, world } from "@minecraft/server";
import { clamp, getOrCreateScoreboard, isBlockLoaded, isUnderGround, isUnderWater, lerp, randomAB } from '../util';
import { Vector } from "../vector";
import { LinearCurve } from "../util/linear_curve";
import { splashTick } from "./splash";
import { GlobalValues } from "./global_values";
import { biomeEffectTick } from "./ambient_effects";
import SimplexNoise from "../util/noise";
import BIOMES from './biomes';
import { isEffectEnabled } from "./settings";
const SUN_LOOKUP_TABLE = new LinearCurve({
    0: 0,
    1000: 710,
    2000: 1490,
    3000: 2300,
    4000: 3200,
    4500: 3670,
    5000: 4160,
    6000: 5160,
    7000: 6100,
});
const MOON_LOOKUP_TABLE = new LinearCurve({
    0: 0,
    2000: 2370,
    4000: 4660,
    5200: 5980,
    6000: 6400,
});
let lens_flare_smooth_y = {};
let lens_flare_smooth_opacity = {};
const CloudNoise = new SimplexNoise();
const WindNoise = new SimplexNoise();
let worldtime_scoreboard = getOrCreateScoreboard('SS_worldtime');
let slow_tick = 0;
export function vfxTick() {
    let overworld = world.getDimension('overworld');
    let the_end = world.getDimension('the_end');
    let old_absolute_time = GlobalValues.absolute_time;
    GlobalValues.time_of_day = world.getTimeOfDay();
    GlobalValues.absolute_time = world.getAbsoluteTime();
    worldtime_scoreboard.setScore('daytime', Math.round(GlobalValues.time_of_day));
    GlobalValues.wind_intensity = clamp(0.8 + WindNoise.noise(GlobalValues.time_of_day / 700, -400), 0.1, 2);
    if (GlobalValues.weather != WeatherType.Clear)
        GlobalValues.wind_intensity += 1.6;
    slow_tick++;
    if (slow_tick == 40) {
        let y_scoreboard = world.scoreboard.getObjective('SS_y_pos');
        let dimension_scoreboard = world.scoreboard.getObjective('SS_dimension');
        world.getAllPlayers().forEach(player => {
            if (!player.isValid())
                return;
            y_scoreboard.setScore(player, Math.round(player.location.y));
            dimension_scoreboard.setScore(player, player.dimension == the_end ? 2 : (player.dimension == overworld ? 0 : 1));
        });
        slow_tick = 0;
    }
    biomeEffectTick();
    if (isEffectEnabled('splashes')) {
        splashTick(overworld);
    }
    // Weather
    if (slow_tick % 3 == 0) {
        if (old_absolute_time && GlobalValues.absolute_time - old_absolute_time > 200) {
            GlobalValues.weather = WeatherType.Clear;
        }
        overworld.getPlayers().forEach(player => {
            if (!player || !player.isValid() || !isBlockLoaded(player.location, overworld))
                return;
            if (player.location.y > 32 && player.location.y < 62 && player.isInWater) {
                player.addTag('in_water');
            }
            else {
                player.removeTag('in_water');
            }
            let biome = player.getProperty('spark:biome_query');
            if (GlobalValues.weather != WeatherType.Clear) {
                if (player.hasTag('particle_spawner_15') && !isUnderGround(player) && !isUnderWater(player)) {
                    if (biome == BIOMES.frozen) {
                        if (slow_tick % 6 == 0 && GlobalValues.weather == WeatherType.Thunder && isEffectEnabled('snowstorms')) {
                            overworld.spawnEntity('spark:blizzard_emitter', player.location);
                        }
                    }
                    else if (biome != BIOMES.desert && biome != BIOMES.savanna && biome != BIOMES.mesa && isEffectEnabled('intense_rain')) {
                        overworld.spawnParticle('spark:light_rain_drizzle', player.location);
                        overworld.spawnParticle('spark:light_wind_gust', player.location);
                    }
                }
            }
        });
        if (GlobalValues.weather != WeatherType.Clear) {
            worldtime_scoreboard.setScore('sunny', -10);
        }
        else {
            worldtime_scoreboard.addScore('sunny', 1);
        }
        let sunny_time = worldtime_scoreboard.getScore('sunny');
        if (sunny_time == -2) {
            if (GlobalValues.time_of_day < 12000 && isEffectEnabled('rainbows')) {
                let direction = GlobalValues.time_of_day < 5900 ? -1 : 1;
                overworld.runCommandAsync(`execute at @r[scores={SS_dimension=0,SS_biome=!2}] positioned ~${200 * direction} ~ ~ run function spark/vfx/events/rainbow`);
            }
        }
        else if (sunny_time == 252) {
            overworld.runCommandAsync('fog @a remove rainbow');
        }
    }
    overworld.getPlayers().forEach(player => {
        if (!player || !player.isValid() || !isBlockLoaded(player.location, overworld))
            return;
        if (GlobalValues.weather != WeatherType.Clear) { // todo: implement rain check in desert
            if (!isUnderGround(player) && !isUnderWater(player) && isEffectEnabled('sandstorms')) {
                let biome = player.getProperty('spark:biome_query');
                if (biome == BIOMES.desert) {
                    if (player.hasTag('particle_spawner_15')) {
                        overworld.spawnParticle('spark:sandstorm', player.location);
                        //overworld.spawnParticle('spark:sandstorm_clouds', player.location);
                        overworld.spawnParticle('spark:sandstorm_sweep', player.location);
                        if (Math.random() < 0.1 && isEffectEnabled('tumbleweeds') && isBlockLoaded(player.location, overworld)) {
                            let tumbleweed = overworld.spawnEntity('spark:tumbleweed', player.location);
                            tumbleweed.addEffect('invisibility', 15, { showParticles: false });
                            tumbleweed.runCommandAsync('spreadplayers ~-30 ~-30 1 40 @s');
                        }
                    }
                }
            }
        }
    });
    if (slow_tick % 10 == 0 && worldtime_scoreboard.getScore('sunny') > 266 && !(worldtime_scoreboard.getScore('northern_lights') >= 1)) {
        let players = overworld.getPlayers({
            tags: ['particle_spawner_40'],
            scoreOptions: [
                { objective: 'SS_y_pos', minScore: 40, maxScore: 300 },
                { objective: 'SS_biome', minScore: 2, maxScore: 2, exclude: true }
            ]
        });
        for (let player of players) {
            if (overworld.getPlayers({ location: player.location, maxDistance: 200, tags: ['in_water'] }).length)
                return;
            if (!isBlockLoaded(player.location, overworld) || !player.isValid())
                return;
            let overall_density = CloudNoise.noise(GlobalValues.absolute_time / 8000, 420);
            let overall_type = CloudNoise.noise(GlobalValues.absolute_time / 14000, -420);
            let overall_texture_val = CloudNoise.noise(GlobalValues.absolute_time / 10000, -1420);
            let squish = overall_type > 0.3 ? 0.6 : (overall_type < -0.5 ? 1.2 : 1);
            let min_radius = 32;
            let type_id = 'spark:clouds_puffy';
            if (overall_texture_val > 0.5) {
                type_id = 'spark:clouds_stratus';
                min_radius = 64;
            }
            else if (overall_texture_val > 0.2) {
                type_id = 'spark:clouds_soft';
            }
            let iterations = Math.round((0.5 + overall_density) * 3.5);
            // Less clouds at night
            if (GlobalValues.time_of_day > 13000)
                iterations -= 2;
            // Regenerate clouds faster when walking
            iterations *= clamp(6 * Math.sqrt(Math.pow(player.getVelocity().x, 2) + Math.pow(player.getVelocity().z, 2)), 1, 3);
            for (let i = 0; i < iterations; i++) {
                let r = (1 - Math.pow(Math.random(), 2)) * 190 + min_radius;
                if (squish < 1)
                    r *= 2;
                let a = Math.random() * Math.PI * 2;
                let pos = [
                    player.location.x + Math.sin(a) * r,
                    player.location.z + Math.cos(a) * r,
                ];
                let shift = (GlobalValues.absolute_time / 1000) * 1.4;
                let noise_val = CloudNoise.noise(pos[0] / 100 + 1 * shift, pos[1] / 100 + 0.4 * shift);
                if (noise_val > randomAB(-0.3, 0.3)) {
                    let variables = new MolangVariableMap();
                    variables.setVector3('cloud_offset', {
                        x: pos[0] - player.location.x,
                        y: 170 + noise_val * 20 - player.location.y,
                        z: pos[1] - player.location.z,
                    });
                    variables.setFloat('squish', squish);
                    overworld.spawnParticle(type_id, player.location, variables);
                }
            }
        }
    }
    // Player Animations + Effects
    if (isEffectEnabled('swimming_trails')) {
        overworld.getPlayers().forEach(player => {
            if (!player || !player.isValid() || !player.isInWater)
                return;
            if (player.isInWater) {
                let block_location = player.location;
                block_location.y += 2.1;
                let block_top;
                try {
                    block_top = overworld.getBlock(block_location);
                }
                catch (err) {
                    return;
                }
                if (!block_top.isValid())
                    return;
                if (block_top.isAir) {
                    let location = {
                        x: player.location.x,
                        y: Math.round(player.location.y + 0.6),
                        z: player.location.z
                    };
                    if (player.isSwimming) {
                        overworld.spawnParticle('spark:swimming_trail', location);
                    }
                    if (Math.random() < 0.2) {
                        location.x += randomAB(-1, 1);
                        location.z += randomAB(-1, 1);
                        overworld.spawnParticle('spark:splash_ripple', location);
                    }
                }
                else if (Math.random() < 0.2) {
                    overworld.spawnParticle('spark:swimming_trail_bubbles', player.location);
                }
            }
        });
    }
    if (slow_tick % 6 == 0) {
        world.getAllPlayers().forEach(player => {
            let moving_boat = player.dimension.getEntities({
                type: 'minecraft:boat',
                maxDistance: 1,
                location: player.location
            }).find(boat => Vector.getLength(boat.getVelocity()) > 0.02);
            if (moving_boat) {
                player.runCommandAsync('playanimation @s animation.spark.player.rowing none 0 "!q.is_riding"');
            }
        });
    }
    // Tumbleweed
    overworld.getEntities({ type: 'spark:tumbleweed' }).forEach(tumbleweed => {
        if (!tumbleweed.isValid())
            return;
        let velocity = tumbleweed.getVelocity();
        if (((velocity.x < 0 || velocity.z < 0) && Math.random() < 0.005) ||
            ((velocity.x == 0 && velocity.z == 0) && Math.random() < 0.001) ||
            tumbleweed.isInWater) {
            overworld.runCommandAsync(`particle spark:tumbleweed_break ${tumbleweed.location.x} ${tumbleweed.location.y} ${tumbleweed.location.z}`);
            tumbleweed.remove();
            return;
        }
        let speed = 0.03 * (tumbleweed.getProperty('spark:wind_effect') || 1) * GlobalValues.wind_intensity;
        if (speed > 0.015) {
            tumbleweed.applyImpulse({
                x: speed * (1 + (Math.random() - 0.5) * 2.0),
                y: 0,
                z: speed * 1.12
            });
        }
    });
    // Lens flare
    let is_day_time = GlobalValues.time_of_day < 12800 || GlobalValues.time_of_day > 23200;
    if (GlobalValues.weather == WeatherType.Clear && ((is_day_time && isEffectEnabled('sun_lens_flare')) || (!is_day_time && isEffectEnabled('moon_lens_flare')))) {
        let flare_angle = 0, variant = 'day';
        if (is_day_time) {
            let sun_time = (((GlobalValues.time_of_day + 6000) % 24000) - 12000);
            flare_angle = SUN_LOOKUP_TABLE.get(Math.abs(sun_time)) * Math.sign(sun_time) / 12000 * Math.PI;
            if (Math.abs(sun_time) > 6000) {
                variant = 'sunset';
            }
        }
        else {
            let moon_time = (((GlobalValues.time_of_day + 6000 + 12000) % 24000) - 12000);
            flare_angle = MOON_LOOKUP_TABLE.get(Math.abs(moon_time)) * Math.sign(moon_time) / 12000 * Math.PI;
            variant = 'moon';
        }
        let transition_speed = 0.03;
        let transition_fade = Math.max(0, 1 - Math.abs(GlobalValues.time_of_day - 12800) * transition_speed, 1 - Math.abs(GlobalValues.time_of_day - 23200) * transition_speed);
        let distance = 55;
        let offset = {
            x: -Math.sin(flare_angle) * distance,
            y: Math.cos(flare_angle) * distance,
            z: 0,
        };
        overworld.getEntities({ type: 'spark:lens_flare' }).forEach(entity => {
            if (!entity.isValid())
                return;
            entity.remove();
        });
        overworld.getPlayers().forEach(player => {
            var _a;
            if (!player.isValid())
                return;
            let eye = {
                x: player.location.x,
                y: player.location.y + (player.isSneaking ? 1.3 : 1.6),
                z: player.location.z,
            };
            eye.y = lerp((_a = lens_flare_smooth_y[player.id]) !== null && _a !== void 0 ? _a : eye.y, eye.y, 0.2);
            lens_flare_smooth_y[player.id] = eye.y;
            if (lens_flare_smooth_opacity[player.id] == undefined)
                lens_flare_smooth_opacity[player.id] = 0;
            let ray = overworld.getBlockFromRay(eye, offset, {
                includeLiquidBlocks: true,
                includePassableBlocks: false,
                maxDistance: 120
            });
            let is_blocked = ray && ray.block && ray.block.isValid() && !ray.block.isAir;
            let target_opacity = (is_blocked ? 0 : 1);
            let opacity = lens_flare_smooth_opacity[player.id] = lerp(lens_flare_smooth_opacity[player.id], target_opacity, 0.5);
            opacity *= (1 - transition_fade);
            if (opacity < 0.05) {
                return;
            }
            try {
                let opacity_step = Math.round(opacity * 20);
                let entity = overworld.spawnEntity(`spark:lens_flare<spark:${variant}_o_${opacity_step}>`, Vector.add(eye, offset));
                if (!entity || !entity.isValid()) {
                    return;
                }
                let velocity = player.getVelocity();
                entity.applyImpulse(velocity);
            }
            catch (err) {
            }
        });
    }
    else {
        overworld.getEntities({ type: 'spark:lens_flare' }).forEach(entity => {
            if (!entity.isValid())
                return;
            entity.remove();
        });
    }
}
export function runBirdEvent(player, position, dimension) {
    let block = dimension.getBlock(position);
    if (!block || !block.isValid() || block.isAir)
        return;
    if (block.permutation.matches('minecraft:grass') || block.permutation.matches('minecraft:leaves') || block.permutation.matches('minecraft:leaves2')) {
        dimension.runCommandAsync(`playsound sound.spark.realism.bird_event @a ${position.x} ${position.y} ${position.z} 2`);
        let variables = new MolangVariableMap();
        let player_offset = [
            player.location.x - position.x,
            player.location.z - position.z,
        ];
        variables.setFloat('rotation', Math.atan2(player_offset[1], -player_offset[0]) / Math.PI * 180 + randomAB(-70, 70));
        dimension.spawnParticle('spark:bird_event', position, variables);
    }
}
world.afterEvents.weatherChange.subscribe((event) => {
    GlobalValues.weather = event.newWeather;
});
world.afterEvents.entityHitEntity.subscribe((event) => {
    if (event.damagingEntity instanceof Player && event.damagingEntity.location.y > 63) {
        let { dimension } = event.hitEntity;
        if (dimension.id != 'minecraft:overworld')
            return;
        if (Math.random() > 0.7)
            return;
        let player = event.damagingEntity;
        let player_dir = player.getViewDirection();
        let offset = {
            x: event.hitEntity.location.x + player_dir.x * 9 + Math.atan(randomAB(-7, 7)) * 7,
            y: event.hitEntity.location.y + randomAB(-3, 3),
            z: event.hitEntity.location.z + player_dir.z * 9 + Math.atan(randomAB(-7, 7)) * 7
        };
        system.runTimeout(() => {
            runBirdEvent(event.damagingEntity, offset, dimension);
        }, 3);
    }
});
world.afterEvents.entitySpawn.subscribe(({ entity, cause }) => {
    if (entity.typeId == 'spark:tumbleweed' && !isEffectEnabled('tumbleweeds') && entity.isValid() && cause == EntityInitializationCause.Spawned) {
        entity.remove();
    }
    if (entity.typeId == 'spark:fly_swarm' && !isEffectEnabled('flies') && entity.isValid() && cause == EntityInitializationCause.Spawned) {
        entity.remove();
    }
});
